; File Name: init.s
;
; Example code from ARM; tailored for the DBMX1 EDP
; This assembly code is used to initialize the
; stack pointers of the ARM core for its different
; modes (i.e. SVC mode, FIQ mode, IRQ mode, etc...)


;;; Copyright ARM Ltd 1999. All rights reserved.
;
; This module performs ROM/RAM remapping (if required), initializes stack pointers and 
; interrupts for each mode, and finally branches to __main in the C library (which 
; eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, with IRQ and FIQ disabled.


        AREA    Init, CODE, READONLY


; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UNDEF      EQU     0x1B
Mode_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled


; --- System memory locations

RAM_Limit       EQU     0x08020000          ; For SDRAM at CSD0

SVC_Stack       EQU     RAM_Limit           ; 6144 byte SVC stack at top of memory
IRQ_Stack       EQU     RAM_Limit-6144       ; followed by IRQ stack
FIQ_Stack		EQU		IRQ_Stack-128		; followed by FIQ stack
; add FIQ_Stack, ABT_Stack, UNDEF_Stack here if you need them
;USR_Stack       EQU     IRQ_Stack-256       ; followed by USR stack
USR_Stack       EQU     FIQ_Stack-128       ; followed by USR stack

;ROM_Start       EQU     0x04000000          ; Base address of ROM after remapping
;Instruct_2      EQU     ROM_Start + 4       ; Address of second instruction in ROM

;ResetBase       EQU     0x0B000000          ; Base address of RPS Remap and Pause Controller
;ClearResetMap   EQU     ResetBase + 0x20    ; Offset of remap control from base


;;        ENTRY

; --- Perform ROM/RAM remapping, if required
;    IF :DEF: ROM_RAM_REMAP

; On reset, an aliased copy of ROM is at 0x0.
; Continue execution from 'real' ROM rather than aliased copy
;        LDR     pc, =Instruct_2

; Remap by writing to ClearResetMap in the RPS Remap and Pause Controller
;        MOV     r0, #0
;        LDR     r1, =ClearResetMap
;        STRB    r0, [r1]

; RAM is now at 0x0.
; The exception vectors (in vectors.s) must be copied from ROM to the RAM
; The copying is done later by the C library code inside __main

; Note (6/15/01): Refer to scatter file, .scf, for info about placing vectors.s
; in memory

;    ENDIF

        EXPORT  Reset_Handler

Reset_Handler

	bl MMU_Setup
	bl PrvFlushCacheAndTLB_AND_DisableCaches
	
;// enable I cache and the MMU
    MRC p15,0,r1,c1,c0,0
    LDR r2, =(0x00001001)	;// enables I cache and MMU, refer to ARM920T Technical Reference Manual
;	LDR r2, =(0x00001005)	;// uncomment this to enable D cache     
    orr r1,r2,r1
    MCR p15,0,r1,c1,c0,0	 
	
; --- Initialise stack pointer registers
; Enter SVC mode and set up the SVC stack pointer
        MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
        LDR     SP, =SVC_Stack

; Enter IRQ mode and set up the IRQ stack pointer
        MSR     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        LDR     SP, =IRQ_Stack

; Enter FIQ mode and set up the FIQ stack pointer
        MSR     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit ; No interrupts
        LDR     SP, =FIQ_Stack


; Set up other stack pointers if necessary
        ; ...

; --- Initialise memory system
        ; ...

; --- Initialise critical IO devices
        ; ...

; --- Initialise interrupt system variables here
        ; ...

; --- Now change to User mode and set up User mode stack.
;        MSR     CPSR_c, #Mode_USR:OR:I_Bit:OR:F_Bit ; No interrupts
;        LDR     SP, =USR_Stack

; Finally, Re-Enter SVC mode
        MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
 

        IMPORT  __main

; --- Now enter the C code
        B      __main   ; note use B not BL, because an application will never return this way

	LTORG		;// place LTORG here, before the MMU translation table, but after all other LDR commands
	        
    GBLA	XCount
       
MMUTranslationTable


;// DBMX1 translation table
XCount	SETA	(0x0800007E)	;// re-map address 0x0 (1MB) to 0x08000000 (SDRAM) 
		DCD	XCount
XCount	SETA	(0x0010007E) 
		DCD	XCount
XCount	SETA	(0x00200072) 
		DCD	XCount		
;XCount	SETA	(0x00300072) ; Use 0x72 for non-cacheable, non-bufferable
XCount	SETA	(0x0030007A) ; Use 0x7A for cacheable only
;XCount	SETA	(0x0030007E) ; Use 0x7E for both cacheable and bufferable. 
	WHILE XCount < (0x10000000)
		DCD	XCount
XCount	SETA	(XCount + 0x00100000)
	WEND

XCount	SETA	(0x1000007A) ; Use 0x7A for cacheable only
	WHILE XCount < (0xFFF00000)		
		DCD	XCount
XCount	SETA	(XCount + 0x00100000)
	WEND

;// 0xFFF00000 to end of memory map
XCount	SETA	(0xFFF0007E)	; Use 0x7E for both cacheable and bufferable.  	
		DCD	XCount
		

EndOfTables

	

MMU_Setup
	LDR	r0, =0x4000	; target address = 0x4000
	
	;// i.MX1 SDRAM location
	add	r0, r0, #0x0B000000;  Physical target address, fourth bank of SDRAM

	ADRL	r1, MMUTranslationTable			; source address
	ADR	r2, EndOfTables					; limit of source

copyloop1
		ldmia	r1!, {r3-r7}
		stmia	r0!, {r3-r7}
		subs	r3, r1, r2
		blt		copyloop1


	LDR	r0, =0x555555f1
	MCR	p15, 0, r0, c3, c0, 0

	;; make the translation table base ptr point to our table
	;// i.MX1 SDRAM location
	LDR	r0, =0x0B004000		;MMUTranslationTableAddr
	
	MCR	p15, 0, r0, c2, c0, 0

	bx	lr  


PrvFlushCacheAndTLB_AND_DisableCaches

	ldr	r1, =0x00000000

	;; Drain the write buffer
;	mcr	p15, 0, r1, c7, c10, 4

	;; Clean and Flush the Cache
	mcr	p15, 0, r1, c7, c7, 0

	;; Drain the write buffer
	mcr	p15, 0, r1, c7, c10, 4

	;; Flush the TLB
	mcr	p15, 0, r1, c8, c7, 0

	;; Read the coprocessor control register
	mrc	p15, 0, r1, c1, c0, 0
	mov	r2, #0x00001000
	add	r2, r2, #0x4
	bic r1, r1, r2
	
	;write to control register
	mcr p15, 0, r1, c1, c0, 0

	BX	lr

        END


